@@ -11,15 +11,30 @@
 # 
 # @keyword manip
 # @arguments number of times to evaluate the expression
-# @arguments expression to evaluation
+# @arguments expression to evaluate
 # @arguments name of the progress bar to use, see \code{\link{create_progress_bar}}
 # @value list of results
 #
 #X mods <- rlply(100, lm(y ~ x, data=data.frame(x=rnorm(100), y=rnorm(100))))
 #X hist(laply(mods, function(x) summary(x)$r.squared))
 rlply <- function(.n, .expr, .progress = "none") {
-  f <- eval.parent(substitute(function(...) .expr))
-  llply(.data = seq_len(.n), .fun = f, .progress = .progress)
+  if (is.function(.expr)) {
+    f <- .expr
+  } else {
+    f <- eval.parent(substitute(function() .expr))    
+  }
+
+  progress <- create_progress_bar(.progress)  
+  result <- vector("list", length = .n)
+
+  progress$init(.n)
+  for(i in seq_len(.n)) {
+    result[[i]] <- f()
+    progress$step()
+  }
+  progress$term()
+  
+  result
 }
 
 # Replicate expression and return results in a data frame
@@ -36,15 +51,23 @@ rlply <- function(.n, .expr, .progress = "none") {
 # 
 # @keyword manip
 # @arguments number of times to evaluate the expression
-# @arguments expression to evaluation
+# @arguments expression to evaluate
 # @arguments name of the progress bar to use, see \code{\link{create_progress_bar}}
 # @value a data frame
 #X
 #X rdply(20, mean(runif(100)))
 #X rdply(20, each(mean, var)(runif(100)))
+#X rdply(20, data.frame(x = runif(2)))
 rdply <- function(.n, .expr, .progress = "none") {
-  f <- eval.parent(substitute(function(...) .expr))
-  ldply(.data = seq_len(.n), .fun = f, .progress = .progress)
+  if (is.function(.expr)) {
+    f <- .expr
+  } else {
+    f <- eval.parent(substitute(function() .expr))    
+  }
+  
+  res <- rlply(.n = .n, .expr = f, .progress = .progress)
+  labels <- data.frame(.n = seq_len(.n))
+  list_to_dataframe(res, labels)
 }
 
 
@@ -62,7 +85,7 @@ rdply <- function(.n, .expr, .progress = "none") {
 # 
 # @keyword manip
 # @arguments number of times to evaluate the expression
-# @arguments expression to evaluation
+# @arguments expression to evaluate
 # @arguments name of the progress bar to use, see \code{\link{create_progress_bar}}
 # @value if results are atomic with same type and dimensionality, a vector, matrix or array; otherwise, a list-array (a list with dimensions)
 #
@@ -76,9 +99,15 @@ rdply <- function(.n, .expr, .progress = "none") {
 #X hist(raply(1000, mean(rexp(10))))
 #X hist(raply(1000, mean(rexp(100))))
 #X hist(raply(1000, mean(rexp(1000))))
-raply <- function(.n, .expr, .progress = "none") {
-  f <- eval.parent(substitute(function(...) .expr))
-  laply(.data = seq_len(.n), .fun = f, .progress = .progress)
+raply <- function(.n, .expr, .progress = "none", .drop = TRUE) {
+  if (is.function(.expr)) {
+    f <- .expr
+  } else {
+    f <- eval.parent(substitute(function() .expr))    
+  }
+  
+  res <- rlply(.n = .n, .expr = f, .progress = .progress)
+  list_to_array(res, NULL, .drop)
 }
 
 # Replicate expression and discard results
@@ -93,12 +122,27 @@ raply <- function(.n, .expr, .progress = "none") {
 # 
 # @keyword manip
 # @arguments number of times to evaluate the expression
-# @arguments expression to evaluation
+# @arguments expression to evaluate
 # @arguments name of the progress bar to use, see \code{\link{create_progress_bar}}
 # @argument automatically print each result? (default: \code{FALSE})
 # 
 #X r_ply(10, plot(runif(50)))
+#X r_ply(25, hist(runif(1000)))
 r_ply <- function(.n, .expr, .progress = "none", .print = FALSE) {
-  f <- eval.parent(substitute(function(...) .expr))
-  l_ply(.data = seq_len(.n), .fun = f, .progress = .progress, .print = .print)
+  if (is.function(.expr)) {
+    f <- .expr
+  } else {
+    f <- eval.parent(substitute(function() .expr))    
+  }
+
+  progress <- create_progress_bar(.progress)  
+
+  progress$init(.n)
+  for(i in seq_len(.n)) {
+    f()
+    progress$step()
+  }
+  progress$term()
+
+  invisible()
 }
\ No newline at end of file
